# Network analysis of MSI data
#
# MIT License
# 
# Copyright (c) 2021 Paolo Inglese
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#   
#   The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# Author: Paolo Inglese (Imperial College London)
#

library(WGCNA)
library(EBImage)
library(reshape2)
library(lattice)
library(gridExtra)
library(rasterVis)
library(tidyverse)
library(ggpubr)
library(betareg)


ANALYSIS <- "no_infused" # It can be 'glucose' or 'glutamine' or 'pdx' or 'no_infused'
# setwd("")  # Set the working directory here

source("./functions.R")

if (ANALYSIS == "glucose") {
  data_dir <- "./DATA/glucose_infused"
} else if (ANALYSIS == "glutamine") {
  data_dir <- "./DATA/glutamine_infused"
} else if (ANALYSIS == "no_infused") {
  data_dir <- "./DATA/no_infused"
} else if (ANALYSIS == "demo") {
  data_dir <- "./DATA/demo"
}
rds_dir <- file.path(data_dir, "_RDS")
if (!dir.exists(rds_dir)) dir.create(rds_dir, recursive = T)
plots_dir <- file.path(data_dir, "_PLOTS")
if (!dir.exists(plots_dir)) dir.create(plots_dir, recursive = T)
csv_dir <- file.path(data_dir, "_CSV")
if (!dir.exists(csv_dir)) dir.create(csv_dir, recursive = T)

msi_dirs <- listMSIDirs(data_dir)
n_runs <- length(msi_dirs)
print(paste0("Num. runs = ", n_runs))

msi_data <- vector(mode = "list", length = n_runs)
for (i in 1:n_runs) {
  msi_data[[i]] <- loadMSI(msi_dirs[i])
  msi_data[[i]]$X <- medNorm(msi_data[[i]]$X)
}

rois <- lapply(1:n_runs, function(i) getROI(msi_data[[i]]$X, msi_data[[i]]$dxy))

# Load the annotated pixels
tissue_labels <- readRDS(file.path(rds_dir, "tissue_labels.rds"))

# Fix batch effect -------------------------------------------------------------

for (i in 1:n_runs) {
  msi_data[[i]]$X <- msi_data[[i]]$X[!is.na(tissue_labels[[i]]), ]
}
msi_data <- fixBatch(msi_data)

# WGCNA ------------------------------------------------------------------------


# Determine soft power
sft <- vector(mode = "list", length = length(msi_data))
for (i in 1:n_runs) {
  filt_labels <- tissue_labels[[i]][!is.na(tissue_labels[[i]])]
  mask <- filt_labels %in% c("lo", "hi")
  sft[[i]] <- WGCNA::pickSoftThreshold(msi_data[[i]]$X[mask, ],
    networkType = "signed hybrid"
  )
  rm(mask, filt_labels)
}
soft.powers <- lapply(sft, function(x) x$fitIndices$Power[x$fitIndices$SFT.R.sq >= 0.85])
soft.power <- min(Reduce(intersect, soft.powers))

# Run WGCNA with only lo/hi myc
net_data <- vector(mode = "list", length = n_runs)
for (i in 1:n_runs) {
  filt_labels <- tissue_labels[[i]][!is.na(tissue_labels[[i]])]
  mask <- filt_labels %in% c("lo", "hi")
  net_data[[i]] <- list(data = msi_data[[i]]$X[mask, ])
  rm(mask, filt_labels)
}

WGCNA::enableWGCNAThreads(nThreads = 10)
modules <- WGCNA::blockwiseConsensusModules(net_data,
  power = soft.power, # This corresponds to the smallest power > R2 in all runs
  networkType = "signed hybrid",
  nThreads = 10
)
disableWGCNAThreads()
closeAllConnections()
gc()
saveRDS(modules, file = file.path(rds_dir, "modules.rds"))

# Save table modules
module_table <- data.frame(
  mz = msi_data[[1]]$cmz, module = modules$colors
)
write.csv(module_table, file = file.path(csv_dir, "modules.csv"))

# Calculate module eigenmetabolites
ME <- computeME(msi_data, tissue_labels, modules$colors)

# Save ME
for (i in 1:n_runs) {
  run_id <- strsplit(msi_dirs[i], "/")[[1]]
  run_id <- run_id[length(run_id)]
  saveRDS(ME[[i]], file = file.path(rds_dir, paste0("ME_", run_id, ".rds")))
}

# Plot ME
if (!dir.exists(file.path(plots_dir, "me")))
  dir.create(file.path(plots_dir, "me"), recursive = T)
sapply(list.files(file.path(plots_dir, "me"), recursive = T),
       function(x) file.remove(x))
for (i in 1:ncol(ME[[1]]$eigengenes)) {
  p <- vector("list", n_runs)

  all_vals <- c()
  for (j in 1:length(msi_data)) {
    all_vals <- c(all_vals, ME[[j]]$eigengenes[, i])
  }
  cuts <- seq(
    min(all_vals, na.rm = T), max(all_vals, na.rm = T),
    (max(all_vals, na.rm = T) - min(all_vals, na.rm = T)) / 63  # Plot 64 colors
  )

  for (j in 1:length(msi_data)) {
    im <- ME[[j]]$eigengenes[, i]
    r <- raster::raster(
      t(matrix(im, msi_data[[j]]$dxy[2:1])),
      xmn = 1, xmx = msi_data[[j]]$dxy[2],
      ymn = 1, ymx = msi_data[[j]]$dxy[1]
    )
    p[[j]] <- rasterVis::levelplot(r, at = cuts, par.settings = infernoTheme)
  }
  pdf(file.path(plots_dir, "me", paste0(colnames(ME[[1]]$eigengenes)[i], ".pdf")),
    width = 12, height = 20
  )
  gridExtra::grid.arrange(grobs = p, top = colnames(ME[[1]]$eigengenes)[i])
  dev.off()
}

# Module selection -------------------------------------------------------------
## Will require multiple replicate runs (n_runs>1)

if(n_runs > 1)
{
  nmodules <- ncol(ME[[1]]$eigengenes)
  name_mods <- colnames(ME[[1]]$eigengenes)
  filt_mods <- name_mods[name_mods != "MEgrey"]
  grey_col <- which(name_mods == "MEgrey")
  
  px_coords <- vector("list", n_runs)
  px_nb <- vector("list", n_runs)
  for (i in 1:n_runs) {
    print(i)
    m <- tissue_labels[[i]] %in% c("lo", "hi")  # Use only lo/hi myc
    rc <- which(matrix(m, nrow = msi_data[[i]]$dxy[2], ncol = msi_data[[i]]$dxy[1]),
      arr.ind = T
    )
    px_coords[[i]] <- rc
  }
  
  fit_data <- data.frame(
    label = character(0),
    run = numeric(0),
    row = numeric(0),
    col = numeric(0),
    id = numeric(0)
  )
  fit_data_me <- matrix(NA,
    nrow = 0, ncol = length(filt_mods),
    dimnames = list(NULL, filt_mods)
  )
  for (i in 1:n_runs) {
    y <- c(tissue_labels[[i]])[which(tissue_labels[[i]] %in% c("lo", "hi"))]
    id <- as.numeric(factor(y)) + (i - 1) * 2
    run <- rep(i, length(y))
    fit_data <- rbind(
      fit_data,
      data.frame(
        label = y,
        run = run,
        row = px_coords[[i]][, 1],
        col = px_coords[[i]][, 2],
        id = id
      )
    )
    fit_data_me <- rbind(fit_data_me,
      me = ME[[i]]$eigengenes[
        which(tissue_labels[[i]] %in% c("lo", "hi")), -grey_col
      ]
    )
  }
  fit_data_me <- as.data.frame(fit_data_me)
  fit_data <- cbind(fit_data, fit_data_me)
  fit_data$label <- factor(fit_data$label)
  fit_data$id <- factor(fit_data$id)
  
  mdls <- vector(mode = "list", length = length(filt_mods))
  names(mdls) <- filt_mods
  for (k in 1:length(mdls))
  {
    curr_me <- filt_mods[k]
    print(curr_me)
    fit_data$binary <- fit_data[, filt_mods[k]] > median(fit_data[, filt_mods[k]])
  
    dframe <- data.frame(
      run = numeric(0),
      id = numeric(0),
      label = character(0),
      ratio = numeric(0)
    )
    for (i in 1:length(msi_data)) {
      subset_ <- subset(fit_data, run == i)
      subset_2 <- subset(subset_, id == 1 + (i - 1) * 2)
      n_hi <- sum(subset_2$binary) / nrow(subset_2)
      subset_2 <- subset(subset_, id == 2 + (i - 1) * 2)
      n_lo <- sum(subset_2$binary) / nrow(subset_2)
  
      dframe <- rbind(
        dframe,
        data.frame(
          ratio = c(n_lo, n_hi),
          label = c("lo", "hi"),
          id = c(1:2) + (i - 1) * 2,
          run = c(i, i)
        )
      )
    }
    mdls[[filt_mods[k]]] <- betareg::betareg(ratio ~ label, data = dframe)
  }
  
  beta1 <- unlist(lapply(mdls, function(x) summary(x)$coefficients$mean[1, 4]))
  pvals <- unlist(lapply(mdls, function(x) summary(x)$coefficients$mean[2, 4]))
  coefs_tab <- data.frame(
    module = names(mdls), coeff = beta1, pval = pvals,
    padj = p.adjust(pvals, "BH")
  )
  rownames(coefs_tab) <- NULL
  write.csv(coefs_tab, file.path(csv_dir, "significance_module_eigengenes_myc.csv"))
  
  # Boxplot proportions ----------------------------------------------------------
  
  p = vector('list', length(fit_data))
  names(p) = names(fit_data)
  for (k in 1:length(mdls)) {
    print(k)
    
    fit_data$binary = fit_data[, filt_mods[k]] > median(fit_data[, filt_mods[k]])
    
    dframe = data.frame(run = numeric(0),
                        id = numeric(0),
                        myc = character(0),
                        ratio = numeric(0))
    for (i in 1:3) {
      subset_ = subset(fit_data, run == i)
      subset_2 = subset(subset_, id == 1 + (i-1) * 2)
      n_hi = sum(subset_2$binary) / nrow(subset_2)
      subset_2 = subset(subset_, id == 2 + (i-1) * 2)
      n_lo = sum(subset_2$binary) / nrow(subset_2)
      
      dframe = rbind(dframe,
                     data.frame(ratio = c(n_lo, n_hi),
                                myc = c('lo', 'hi'),
                                id = c(1:2) + (i-1) * 2,
                                run = c(i, i)))
    }
    
    dframe$id = factor(dframe$id)
    
    p[[k]] = ggpubr::ggerrorplot(
      dframe,
      x = 'myc', y = 'ratio',
      color = 'black',
      add = 'jitter',
      add.params = list(color = 'myc', size=2)
    ) + scale_color_brewer(palette='Set1') +
      ggtitle(filt_mods[k])
  }
  
  png(filename = file.path(plots_dir, 'ME_ratio_median_boxplots.png'),
      w=10, h=12, res=300, u='in')
  ggpubr::ggarrange(plotlist = p, common.legend = T)
  dev.off()
}
