# Circos plot for correlated ions from turquoise module (for WMHigh signature)
#
# MIT License
# 
# Copyright (c) 2021 Avinash Ghanate
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#   
#   The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# Author: Avinash Ghanate (The Francis Crick Institute, London)
#

options(warn = -1)
rm(list = ls())
library(hdf5r)
library(tidyverse)
library(WGCNA)

# Define which analysis folder to use -------------------------------------

# setwd("")  # Set the working directory here

source("./functions.R")

data_dir <- "./DATA/no_infused"
plots_dir <- file.path(data_dir, "_PLOTS")
if (!dir.exists(plots_dir)) dir.create(plots_dir, recursive = T)


# List all directories with MSI data --------------------------------------

fileN <- listMSIDirs(data_dir)


# Read data and prepare it for plotting -----------------------------------

totalData <- tibble(fileN) %>% 
  mutate(fileIndx = seq_len(n())) %>% 
  mutate(h5fc = map(fileN, ~h5file(paste0(.x,"/intensity_mat_multi.h5"), mode = "r"))) %>% 
  mutate(mz = map(h5fc, ~.x[["cmz"]]$read()),
         dim_xy = map(h5fc, ~.x[["dim_xy"]]$read()),
         allData = map(h5fc, ~.x[["X"]]),
         tissue_idx = map(allData, ~which(colSums(.x$read())!=0)),
         data = map2(allData, tissue_idx, ~.x$read() %>% t() %>% as.data.frame() %>% slice(.y)),
         all_pixels = pmap(list(dim_xy, fileIndx, fileN), function(a,b,c) which(matrix(1, nrow=a[1], ncol = a[2]) == 1, T) %>% as.data.frame() %>% rename(x = row, y = col) %>% mutate(fileIndx = b, fileN = c))
  ) %>% 
  mutate(
    indx = map2(all_pixels, tissue_idx, ~.x %>% as.data.frame() %>% slice(.y)))

outData <- totalData %>% 
  # Comment this if you dont want normalization
  mutate(data = map(data, ~normData(.x, norm = "median") %>% as.data.frame())) %>%
  summarise(data = list(bind_rows(data)), indx = list(bind_rows(indx)), mz = list(mz[[1]]))
outData <- list(data = outData$data[[1]], mz = outData$mz[[1]], indx = outData$indx[[1]])



# Preparing data for Circos plot ------------------------------------------

# Read annotated data for the WMHigh module
MEdata <- read.csv("Turquoise_HMDB_hits.csv", na.strings = c("","NA")) %>% 
  arrange(Class) 


i= "turquoise"
min.cor = 0.7

print(paste("Module:",i))

# Select ions for specific module and filter putative isotopic ions based upon empirically identified 0.9 cosine correlation cut-off
# between precurson ion intensity and the heavier isotopic ion intensity profile 
mod.Indx <- MEdata %>% 
  mutate(X = as.numeric(gsub("Column\\.", "", X))) %>% 
  filter(module == i) %>% 
  filter(!isoCor) %>% 
  pull(X) %>% 
  unique()

# Retrieve module membership for selected ions
mod.membership <- MEdata %>% 
  filter(module == i) %>% 
  filter(!isoCor) %>% 
  distinct(mz, module_membership) %>% 
  pull(module_membership) 

# Retrieve ion m/z for selected ions
mod.mz <- MEdata %>% 
  filter(module == i) %>% 
  filter(!isoCor) %>% 
  distinct(mz, module_membership) %>% 
  pull(mz) %>% 
  round(digits = 4)

# Retrieve metabolite class based on HMDB based putative annotation for ions using 15 PPM m/z criteria for M-H and M-H2O-H adducts
mz.class <- MEdata %>% as.data.frame() %>% 
  filter(module == i) %>% 
  filter(!isoCor) %>% 
  mutate(dupl = duplicated(cbind(mz))) %>% 
  slice(-which(dupl & adduct=="M-H20-H")) %>% 
  mutate(n = seq_len(nrow(.))) %>% 
  group_nest(mz) %>% 
  mutate(Class = unlist(map(data, ~paste(.x$Class[[1]])))) %>% 
  mutate(n = unlist(map(data, ~head(.x$n, n = 1)))) %>%
  select(-data, -mz) %>% 
  arrange(n) %>% 
  pull(Class) %>% 
  as.character()


# Generate pairwise correlation matrix for selected ions from input data
ions.cor <- outData$data %>% 
  select(mod.Indx) %>% 
  mutate(fileIndx = outData$indx$fileIndx) %>% 
  group_nest(fileIndx) %>% 
  mutate(corRes = map(data,~cor(.x, use = "p") %>% as.data.frame)) %>% 
  select(-data) %>% 
  unnest(c(fileIndx, corRes)) %>% 
  mutate(rowN = rep(seq_len(ncol(.)-1), length(unique(fileIndx)))) %>%
  select(-fileIndx) %>% 
  pivot_longer(-rowN, names_to = "colN", values_to = "cor") %>% 
  pivot_wider(names_from = rowN, values_from = cor, values_fn = list(cor = median)) %>% 
  select(-colN) %>% 
  as.matrix()

# User specified filters to show connection edge between ions for given criteria
if(length(which(abs(ions.cor)>min.cor)) != nrow(ions.cor))
  ions.cor[abs(ions.cor)<min.cor] <- 0

title <- paste0(
  "Module ", i, " (size: ",length(mod.Indx),"), cor. > ", min.cor)


# Plot Circos plot --------------------------------------------------------

pdf(file.path(plots_dir, "Turquoise_circos_plot.pdf"),
    height = 12, width = 12)
edgeGraph(ions.cor, mod.mz, mod.membership, mz.class) + ggtitle(title) +
  theme(plot.title = element_text(hjust = 0.5))
dev.off()
