# Normalization of NanoSIMS data to correct for batch effect
#
# MIT License
# 
# Copyright (c) 2021 Peter Kreuzaler
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#   
#   The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# Author: Peter Kreuzaler (The Francis Crick Institute, London)
#

# This is a script to undergo normalisation of NanoSIMS data acquired on 
# different days and consecutive sections, where the normalisation factor
# is determined by comparison of the distribution between the values in the 
# same small ROI across the two sections. This was mainly necessary for long 
# runs and in particular when looking at 13C

library(tidyverse)
library(ggthemes)
library(extremevalues)
library(ggrepel)
library(ggpubr)
library(hrbrthemes)
library(gridExtra)
library(grid)
library(RColorBrewer)
library(ggsci)

# Functions ---------------------------------------------------------------

get_sd <- function(n, z){
  c_old_sd <- n %>% filter(sample == m[z], !is.na(pixel_value))
  x <- getOutliersI(c_old_sd$pixel_value)
  y <- c(x$iRight, x$iLeft)
  if(length(y) == 0){c_old_sd <- c_old_sd$pixel_value
  } else{
  c_old_sd <- c_old_sd$pixel_value[-y]}
  mad(c_old_sd)
}

get_mean <- function(n, w){
  c_old_mean <- n %>% filter(sample == m[w], !is.na(pixel_value))
  x <- getOutliersI(c_old_mean$pixel_value)
  y <- c(x$iRight, x$iLeft)
  if(length(y) == 0){c_old_mean <- c_old_mean$pixel_mad_norm
  } else{
    c_old_mean <- c_old_mean$pixel_mad_norm[-y]}
  mean(c_old_mean)
}
 



# START -------------------------------------------------------------------

# setwd("")  # Set the working directory here

# Load the file with all pixel values for every imaged tile (NB same script used for 15N and 13C)
ROI_data_15N <- read.csv("2021_02_19_ROI_13_c_comparison_each_tile.csv")

# Load file with average single cell values. Tile for each cell annotated for later correction
measure_15N <- read_csv("2021-02-09 13 C Cell wise comparison.csv")

# plots of the data before normalising

ROI_data_15N %>% ggplot(aes(pixel_value, fill = sample)) +
  geom_density(alpha = 0.2)+
  theme(legend.position = "none")


normalisation <- ROI_data_15N 
nanosims_data <- measure_15N
m <-  levels(normalisation$sample) 
my_comparison <- list(c("Red_mix", "Green_mix"))


# remove outliers and calculate sd

all_sd <- vector(length = length(m))
for(z in 1:length(m)){
  all_sd[z] <- get_sd(normalisation, z)
  
}
all_sd <- data.frame(sample = m, mad = all_sd)


# normalising for sd

normalisation <- normalisation %>%
  left_join(all_sd, by="sample") %>%
  mutate(pixel_mad_norm =
           normalisation$pixel_value*abs((all_sd$mad[all_sd$sample=="190822-SEM01-E3A1D2-60um"]/mad)))


# Calculate mean

all_mean <- vector(length = length(m))
for(w in 1:length(m)){
  all_mean[w] <- get_mean(normalisation, w)
}
all_mean <- data.frame(sample = m, mean = all_mean)


# normalising for mean

normalisation <- normalisation %>%
  left_join(all_mean) %>%
  mutate(pixel_mean_norm =
           normalisation$pixel_mad_norm + (all_mean$mean[all_mean$sample=="190822-SEM01-E3A1D2-60um"]- mean))

# plotting

normalisation %>% 
  ggplot(aes(pixel_mean_norm, fill = sample)) +
  geom_density(alpha = 0.2) +
  theme(legend.position = "none")


normalisation %>% 
  ggplot(aes(x = sample, y = pixel_mean_norm, color = sample)) +
  geom_boxplot() +
  theme(legend.position = "none")
remove(z)
remove(w)


# applying the values to the single cell data -----------------------------

# SD adjustment

nanosims_data <- nanosims_data %>% 
  left_join(all_sd, by="sample") %>%
  mutate(ND_mad_norm =
           nanosims_data$ND*abs((all_sd$mad[all_sd$sample=="190822-SEM01-E3A1D2-60um"]/mad)))


# mean adjustmen

nanosims_data <- nanosims_data %>% 
  left_join(all_mean) %>%
  mutate(ND_mean_norm =
           nanosims_data$ND_mad_norm + (all_mean$mean[all_mean$sample=="190822-SEM01-E3A1D2-60um"]- mean))

nanosims_data %>% group_by(Region) %>% summarize(average = mean(ND_mean_norm))

# Plot

nanosims_data %>% 
  ggplot(aes(ND_mean_norm, fill = Region)) +
  geom_density(alpha = 0.2) + 
  xlim(0,200)

nanosims_data$Region <- factor(nanosims_data$Region, levels = c("Red_mix", "Green_mix"))

image <- nanosims_data %>%
  ggplot(aes(x = Region, y=ND_mean_norm, colour=Region, fill = Region)) +
  geom_boxplot(outlier.colour = NA, size=1, alpha = 0.2, show.legend = FALSE, width = 0.5) +
  geom_point(aes(colour = Region), size = 2, shape = 21, 
             position = position_jitterdodge(), stroke = 1, show.legend = FALSE) +
  scale_fill_manual(values=c("#E64B35B2", "#00A087B2", "#4DBBD5B2")) +
  scale_color_manual(values=c("#E64B35B2", "#00A087B2", "#4DBBD5B2")) +
  stat_compare_means(aes(label = ..p.signif..), method = "t.test", comparisons = my_comparison, step.increase = 0,
                     hide.ns = TRUE, vjust = 0.5, size = 8, show.legend = FALSE, method.args = list(var.equal = TRUE))+
  ggtitle("Glucose per cell") +
  xlab("") + 
  ylab("N/D") +
  theme_clean(base_size = 35)

ggsave(filename = "NanoSIMS_cells_13C_tile corrected.svg", width=4, height=8)

nanosims_data %>%
  ggplot(aes(y=ND, x= Region, fill=Region)) +
  geom_violin() +
  geom_point(aes(colour = Region), size = 0.2, shape = 1, alpha = 0.2,
             position = position_jitterdodge(), stroke = 1, show.legend = FALSE) +
  scale_color_npg()+
  scale_fill_npg(alpha = 0.6)+
  stat_compare_means(aes(label = ..p.signif..), method = "t.test", comparisons = my_comparison, step.increase = 0,
                     hide.ns = TRUE, vjust = 0.5, size = 8, show.legend = FALSE, method.args = list(var.equal = TRUE)) +
  theme_clean(base_size = 20)


# Below is the calculation for the cells, but divided by +/- Brdu
mycomparison2 <-list(c("Red_mix.brdu-", "Green_mix.brdu-"), c("Red_mix.brdu+", "Green_mix.brdu+"))
nanosims_data$subregion <- interaction(nanosims_data$Region, nanosims_data$brdu)
nanosims_data$Region <- factor(nanosims_data$Region, levels = c("Red_mix_brdu-", "Green_mix_brdu-", "Red_mix_brdu+", "Green_mix_brdu+"))

image <-nanosims_data %>% 
  ggplot(aes(x = subregion, y=ND_mean_norm, colour=subregion, fill = subregion)) +
  geom_boxplot(outlier.colour = NA, size=1, alpha = 0.2, show.legend = FALSE) +
  geom_point(aes(colour = subregion), size = 2, shape = 21, 
             position = position_jitterdodge(), stroke = 1, show.legend = FALSE) +
  scale_fill_npg(alpha = 0.2)+
  stat_compare_means(aes(label = ..p.signif..), method = "t.test", comparisons = mycomparison2, step.increase = 0,
                     hide.ns = TRUE, vjust = 0.5, size = 8, show.legend = FALSE, method.args = list(var.equal = TRUE))+
  ggtitle("Glucose per cell") +
  xlab("") + 
  ylab("N/D") +
  theme_clean(base_size = 12)

ggsave(filename = "NanoSIMS_cells_13C_tile corrected_Brdu.svg", width=12, height=8)
