library(ggplot2)
library(dplyr)
library(magrittr)
library(nlme)
library(ggeffects)

library(sjPlot)
library(sjmisc)
library(sjlabelled)

# Load mean PA per tissue
dat <- read.csv('mean_pa_pdx_2Mar2023.csv') %>% 
  rename(Run = run, Tissue = tissue)

# Fit lme
mdl <- lme(Mean.PA ~ Myc, random = ~ 1 | Run / Tissue,
           data = as.data.frame(dat))

# Save diagnostics plot
plist = plot_model(mdl, type='diag')
p <- plot_grid(plist)
ggsave(filename = 'diagn_PA_pdx.pdf',
       plot = p, dpi = 300, width = 8, height = 8)

# Ext Fig 2-g

# Predict fixed effects
pred <- ggpredict(mdl, type='fe', terms=c('Myc'))

# Assign label 5 to 8 for run 2 tissues
dat_plot <- dat
dat_plot$Tissue <- as.numeric(dat_plot$Tissue)
dat_plot$Tissue[dat_plot$Run == 2] <- dat_plot$Tissue[dat_plot$Run == 2] + 4
dat_plot$Tissue <- factor(dat_plot$Tissue)
dat_plot$Run <- factor(dat_plot$Run)

p <-  ggplot() +
  scale_x_discrete() +
  geom_point(aes(x=factor(Myc), y=Mean.PA, color=Tissue, shape=Run), size=5, 
             data=dat_plot, stroke=2) +
  scale_shape_manual(values = c(`1`=1, `2`=2)) +
  geom_line(aes(x=factor(Myc), y=Mean.PA, group=Tissue, color=Tissue),
            data=dat_plot,
            size = 1) +
  scale_color_brewer(palette='Dark2') +
  geom_point(aes(x = x, y = predicted), size=7, color='black', data=pred) +
  geom_errorbar(aes(x=x, ymin=conf.low, ymax=conf.high), color='black', 
                data=pred, width=0.1, size=1) +
  labs(x='Myc Level') +
  theme_bw()
ggsave(filename = 'preds_PA_pdx.pdf', plot = p, dpi = 300, width = 4, 
       height = 4)


# Save fixed effects
colnames(pred)[1] <- 'Myc'
as.data.frame(pred[, -6]) %>%
  write.csv('fixed_effect_myc.csv')

# Save random effects
pred <- ggpredict(mdl, type='re', terms=c('Myc', 'Tissue'))
colnames(pred)[1] <- 'Myc'
colnames(pred)[6] <- 'Tissue'
as.data.frame(pred) %>%
  write.csv('random_effects_within_tissue.csv')