# Plot Single Ion Images for metabolic ions of interest
#
# MIT License
# 
# Copyright (c) 2021 Avinash Ghanate
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#   
#   The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# Author: Avinash Ghanate (The Francis Crick Institute, London)
#

options(warn = -1)
rm(list = ls())
library(hdf5r)
library(tidyverse)

# Define which analysis folder to use -------------------------------------

ANALYSIS <- "demo" # It can be 'glucose' or 'glutamine' or 'pdx' or 'no_infused'
# setwd("")  # Set the working directory here

source("functions.R")

if (ANALYSIS == "glucose") {
  data_dir <- "./DATA/glucose_infused"
} else if (ANALYSIS == "glutamine") {
  data_dir <- "./DATA/glutamine_infused"
} else if (ANALYSIS == "pdx") {
  data_dir <- "DATA/pdx_infused"
} else if (ANALYSIS == "no_infused") {
  data_dir <- "./DATA/no_infused"
} else if (ANALYSIS == "demo") {
  data_dir <- "./DATA/demo"
}

plots_dir <- file.path(data_dir, "_PLOTS")
if (!dir.exists(plots_dir)) dir.create(plots_dir, recursive = T)


# List all directories with MSI data --------------------------------------

fileN <- listMSIDirs(data_dir)


# Read data and prepare it for plotting -----------------------------------

totalData <- tibble(fileN) %>% 
  mutate(fileIndx = seq_len(n())) %>% 
  mutate(h5fc = map(fileN, ~h5file(paste0(.x,"/intensity_mat_multi.h5"), mode = "r"))) %>% 
  mutate(mz = map(h5fc, ~.x[["cmz"]]$read()),
         dim_xy = map(h5fc, ~.x[["dim_xy"]]$read()),
         allData = map(h5fc, ~.x[["X"]]),
         tissue_idx = map(allData, ~which(colSums(.x$read())!=0)),
         data = map2(allData, tissue_idx, ~.x$read() %>% t() %>% as.data.frame() %>% slice(.y)),
         all_pixels = pmap(list(dim_xy, fileIndx, fileN), function(a,b,c) which(matrix(1, nrow=a[1], ncol = a[2]) == 1, T) %>% as.data.frame() %>% rename(x = row, y = col) %>% mutate(fileIndx = b, fileN = c))
  ) %>% 
  mutate(
    indx = map2(all_pixels, tissue_idx, ~.x %>% as.data.frame() %>% slice(.y)))

outData <- totalData %>% 
  # Comment this if you dont want normalization
  mutate(data = map(data, ~normData(.x, norm = "median") %>% as.data.frame())) %>%
  summarise(data = list(bind_rows(data)), indx = list(bind_rows(indx)), mz = list(mz[[1]]))
outData <- list(data = outData$data[[1]], mz = outData$mz[[1]], indx = outData$indx[[1]])


# Plot single ion image for metabolite of interest ------------------------

ions <- tibble(
  mz = c(771.51816, 738.50793, 218.10340),
  name=c("PG 36:3 [M-H]-", "PE 36:4 [M-H]-", "Pantothenic acid [M-H]-")
)

pdf(file.path(plots_dir, "Single ions images.pdf"),
    onefile = TRUE,
    width = 10)
map(plotMSI(dataX = outData$data, indx = outData$indx,mzlist = outData$mz, mz = ions$mz, ppm = 10, Title = ions$name,
            revBG = F, norm = "max"), ~.x + facet_wrap(vars(fileIndx), ncol = 1))
dev.off()

